﻿using System;
using System.Collections.Generic;
using net.citrusleaf;

namespace Citrusleaf.Demo
{
    class BatchGet : Example
    {
        public BatchGet(Console console)
            : base(console)
        {
        }

        /// <summary>
        /// Batch multiple gets in one call to the server.
        /// </summary>
        public override void RunExample(CitrusleafClient cs, string host, int port, string ns, string set, bool singleBin)
        {
            int size = 8;
            string keyPrefix = "batchkey";
            string bin = singleBin ? "" : "demobin";  // Single bin servers don't need a bin name.
            string valuePrefix = "batchvalue";
            writeRecords(cs, ns, set, keyPrefix, bin, valuePrefix, size);
            batchReads(cs, ns, set, keyPrefix, bin, size);
        }

        private void writeRecords
        (
            CitrusleafClient cs, 
            string ns, 
            string set,
            string keyPrefix,
            string bin,
            string valuePrefix,
            int size
        )
        {
            for (int i = 1; i <= size; i++)
            {
                string key = keyPrefix + i;
                string value = valuePrefix + i;
                CitrusleafResult r = cs.Set(ns, set, key, bin, value, null);

                if (r != CitrusleafResult.OK)
                    throw new Exception(string.Format(
                        "Failed to set: ns={0} set={1} key={2} bin={3} value={4} code={5}",
                        ns, set, key, bin, value, r));

                console.Info("Set successful: ns={0} set={1} key={2} bin={3} value={4}",
                    ns, set, key, bin, value);
            }
        }

        private void batchReads
        (
            CitrusleafClient cs, 
            string ns, 
            string set,
            string keyPrefix,
            string bin,
            int size
        )
        {
            // Batch gets into one call.
            List<object> keys = new List<object>();
            for (int i = 1; i <= size; i++)
            {
                keys.Add(keyPrefix + i);
            }

            object[] values;
            uint[] generations;
            CitrusleafResult[] results = cs.BatchGet(ns, set, keys, bin, out values, out generations, DEFAULT_TIMEOUT_MS);

            foreach (CitrusleafResult r in results)
            {
                if (r != CitrusleafResult.OK)
                    throw new Exception(string.Format(
                        "Failed to get: ns={0} set={1} code={2}",
                        ns, set, r));
            }
            console.Info("Get successful:");

            for (int i = 0; i < values.Length; i++)
            {
                object key = keys[i];
                object value = values[i];
                uint generation = generations[i];

                console.Info("ns={0} set={1} key={2} bin={3} value={4}",
                    ns, set, key, bin, value);
            }
        }
    }
}
