﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using net.citrusleaf;

namespace Citrusleaf.Demo
{
    abstract class Example
    {
        protected const int DEFAULT_TIMEOUT_MS = 10000;

        protected Console console;
        protected volatile bool valid;

        public Example(Console console)
        {
            this.console = console;
        }

        public void Run(string host, int port, string ns, string set)
        {
            console.Clear();
            console.Info("{0} Begin", this.GetType().Name);
            valid = true;

            try
            {
                bool singleBin = GetIsSingleBin(host, port, ns);

                CitrusleafClient cs = new CitrusleafClient();

                try
                {
                    cs.SetLogging(CitrusleafClient.LogLevel.INFO, LogCallback, null);

                    if (cs.AddHost(host, port) != CitrusleafResult.OK)
                    {
                        throw new Exception(string.Format("Failed to add {0}:{1} to cluster.", host, port));
                    }
                    RunExample(cs, host, port, ns, set, singleBin);
                }
                finally
                {
                    cs.Dispose();
                }
            }
            catch (Exception ex)
            {
                console.Error(ex.Message);
            }
            console.Info("{0} End", this.GetType().Name);
        }

        public bool GetIsSingleBin(string host, int port, string ns)
        {
            ClLog log = new ClLog();
            log.Set(CitrusleafClient.LogLevel.INFO, LogCallbackPrivate, null);

            string filter = "namespace/" + ns;
            string tokens;
            CitrusleafResult r = CitrusleafClient.Info(host, port, filter, out tokens, log, null);

            if (r != CitrusleafResult.OK)
                throw new Exception(string.Format(
                    "Failed to get namespace info: host={0} port={1} namespace={2} code={3}",
                    host, port, ns, r));

            string name = "single-bin";
            string search = name + '=';
            int begin = tokens.IndexOf(search);

            if (begin < 0)
                throw new Exception(string.Format(
                    "Failed to find namespace attribute: host={0} port={1} namespace={2} attribute={3} code={4}",
                    host, port, ns, name, r));

            begin += search.Length;
            int end = tokens.IndexOf(';', begin);

            if (end < 0)
            {
                end = tokens.Length;
            }
            string value = tokens.Substring(begin, end - begin);
            return Boolean.Parse(value);
        }

        public bool GetIsStopWrites(string host, int port, string ns)
        {
            ClLog log = new ClLog();
            log.Set(CitrusleafClient.LogLevel.INFO, LogCallbackPrivate, null);

            string filter = "namespace/" + ns;
            string tokens;
            CitrusleafResult r = CitrusleafClient.Info(host, port, filter, out tokens, log, null);

            if (r != CitrusleafResult.OK)
                return false;

            string name = "stop-writes";
            string search = name + '=';
            int begin = tokens.IndexOf(search);

            if (begin < 0)
                return false;

            begin += search.Length;
            int end = tokens.IndexOf(';', begin);

            if (end < 0)
            {
                end = tokens.Length;
            }
            string value = tokens.Substring(begin, end - begin);
            return Boolean.Parse(value);
        }

        private void LogCallbackPrivate(CitrusleafClient.LogLevel level, string message, object uobject)
        {
            console.Log(level, message);
        }

        public abstract void RunExample(CitrusleafClient cs, string host, int port, string ns, string set, bool singleBin);

        protected void LogCallback(CitrusleafClient.LogLevel level, string message, object uobject)
        {
            console.Log(level, message);
        }

        public void Stop()
        {
            valid = false;
        }
    }
}
