﻿using System;
using System.Collections.Generic;
using net.citrusleaf;

namespace Citrusleaf.Demo
{
    class Scan : Example, CitrusleafClient.IScanCallback
    {
        private int recordCount = 0;

        public Scan(Console console)
            : base(console)
        {
        }

        /// <summary>
        /// Read all records in a set.
        /// </summary>
        public override void RunExample(CitrusleafClient cs, string host, int port, string ns, string set, bool singleBin)
        {
            console.Info("Scan database set: " + set);
            recordCount = 0;
            DateTime begin = DateTime.Now;
            CitrusleafResult r = cs.Scan(ns, set, -1, this);

            if (r != CitrusleafResult.OK)
            {
                if (r == CitrusleafResult.SCAN_TERMINATED)
                {
                    console.Info("Scan terminated.");
                }
                else
                {
                    throw new Exception(string.Format(
                        "Failed to scan: ns={0} set={1} code={2}",
                        ns, set, r));
                }
            }
            DateTime end = DateTime.Now;
            TimeSpan ts = end.Subtract(begin);
            console.Info("Total records returned: " + recordCount);
            console.Info("Elapsed time: " + ts.ToString());
            double performance = Math.Round(((double)recordCount) / ((double)ts.TotalSeconds), 0);
            console.Info("Records/second: " + performance);
        }

        public bool ScanRecordCallback(string ns, string set, byte[] digest, IDictionary<string, object> values, uint generation, uint expirationDate)
        {
            recordCount++;

            if ((recordCount % 10000) == 0)
            {
                console.Info("Records " + recordCount);
            }
            return valid;
        }
    }
}
