﻿using System;
using System.Collections.Generic;
using net.citrusleaf;

namespace Citrusleaf.Demo
{
    class ScanSeries : Example, CitrusleafClient.IScanCallback
    {
        private Dictionary<string, Metrics> setMap = new Dictionary<string, Metrics>(10);

        public ScanSeries(Console console)
            : base(console)
        {
        }

        /// <summary>
        /// Scan all nodes in series and read all records in all sets.
        /// </summary>
        public override void RunExample(CitrusleafClient cs, string host, int port, string ns, string set, bool singleBin)
        {
            console.Info("Scan series: namespace=" + ns + " set=" + set);

            List<Node> nodeList = cs.GetActiveNodes();
            DateTime begin = DateTime.Now;

            foreach (Node node in nodeList)
            {
                console.Info("Scan node: " + node.name);
                CitrusleafResult r = cs.ScanNode(node, ns, set, this);

                if (r != CitrusleafResult.OK)
                {
                    if (r == CitrusleafResult.SCAN_TERMINATED)
                    {
                        console.Info("Scan terminated.");
                    }
                    else
                    {
                        throw new Exception(string.Format(
                            "Failed to scan: ns={0} set={1} node={2} code={3}",
                            ns, set, node.name, r));
                    }
                }

                foreach (KeyValuePair<string, Metrics> entry in setMap) 
                {
                    console.Info("Node " + node.name + " set " + entry.Key + " count: " + entry.Value.count);
				    entry.Value.count = 0;
                }
            }

            DateTime end = DateTime.Now;
            TimeSpan ts = end.Subtract(begin);
            console.Info("Elapsed time: " + ts.ToString());

            long total = 0;

            foreach (KeyValuePair<string, Metrics> entry in setMap)
            {
                console.Info("Total set " + entry.Key + " count: " + entry.Value.total);
                total += entry.Value.total;
            }
            console.Info("Grand total: " + total);
            double performance = Math.Round((double)total / ts.TotalSeconds);
            console.Info("Records/second: " + performance);
        }

        public bool ScanRecordCallback(string ns, string set, byte[] digest, IDictionary<string, object> values, uint generation, uint expirationDate)
        {
            Metrics metrics = null;

            if (! setMap.TryGetValue(set, out metrics))
            {
                metrics = new Metrics();
                setMap[set] = metrics;
            }
            metrics.count++;
            metrics.total++;
            return true;
        }
    }

    class Metrics
    {
        public long count = 0;
        public long total = 0;
    }
}
