﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using net.citrusleaf;

namespace Citrusleaf.Demo
{
    class Serialize : Example
    {
        public Serialize(Console console)
            : base(console)
        {
        }

        /// <summary>
        /// Write complex object using serializer.
        /// </summary>
        public override void RunExample(CitrusleafClient cs, string host, int port, string ns, string set, bool singleBin)
        {
            string key = "serializetestserializetestserializetest";
            string bin = singleBin ? "" : "serialbin";  // Single bin servers don't need a bin name.

            // Delete record if it already exists.
            CitrusleafResult r = cs.Delete(ns, set, key, null);

            if (!(r == CitrusleafResult.OK || r == CitrusleafResult.KEY_NOT_FOUND_ERROR))
                throw new Exception(string.Format("Failed to delete: ns={0} set={1} key={2} code={3}",
                    ns, set, key, r));

            console.Info("Initialize array");

            int[] array = new int[10000];

            for (int i = 0; i < 10000; i++) 
            {
                array[i] = i * i;
            }

            // Do a test that pushes this complex object through the serializer.
            console.Info("Write array using serializer.");
            r = cs.Set(ns, set, key, bin, array, null);

            if (r != CitrusleafResult.OK)
                throw new Exception(string.Format(
                    "Failed to set: ns={0} set={1} key={2} bin={3} value={4} code={5}",
                    ns, set, key, bin, array, r));

            console.Info("Read array using serializer.");
            object received;
            uint generation;
            r = cs.Get(ns, set, key, bin, out received, out generation, DEFAULT_TIMEOUT_MS);

            if (r != CitrusleafResult.OK)
                throw new Exception(string.Format(
                    "Failed to get: ns={0} set={1} key={2} bin={3} code={4}",
                    ns, set, key, bin, r));

            int[] receivedArray;

            try 
            {
                receivedArray = (int[])received;
            }
            catch (Exception e) 
            {
                throw new Exception(String.Format(
                    "Failed to parse returned value: namespace={0} set={1} key={2} bin={3} exception={4}", 
                    ns, set, key, bin, e.Message));
            }

            if (receivedArray.Length != 10000) 
            {
                throw new Exception(String.Format(
                    "Array length mismatch: Expected=%d Received=%d", 10000, receivedArray.Length));
            }

            for (int i = 0; i < 10000; i++) 
            {
                if (receivedArray[i] != i * i)
                {
                    throw new Exception(String.Format(
                        "Mismatch: index=%d expected=%d received=%d", i, receivedArray[i], i*i)); 
                }
            }
            console.Info("Read successful.");
        }
    }
}
