﻿using System;
using System.Collections.Generic;
using net.citrusleaf;

namespace Citrusleaf.Demo
{
    class ServerInfo : Example
    {
        public ServerInfo(Console console)
            : base(console)
        {
        }

        /// <summary>
        /// Query server configuration, cluster status and namespace configuration.
        /// </summary>
        public override void RunExample(CitrusleafClient cs, string host, int port, string ns, string set, bool singleBin)
        {
            ClLog log = new ClLog();
            log.Set(CitrusleafClient.LogLevel.INFO, LogCallback, null);

            GetServerConfig(host, port, log);
            console.Log("");
            GetNamespaceConfig(host, port, ns, log);
        }

         /// <summary>
        /// Query server configuration and cluster status.
        /// </summary>
        private void GetServerConfig(string host, int port, ClLog log)
        {
            console.Log("Server Configuration");
            IDictionary<string, string> info = new Dictionary<string, string>();
            CitrusleafResult r = CitrusleafClient.Info(host, port, out info, log, null);

            if (r != CitrusleafResult.OK)
                throw new Exception(string.Format("Failed to get server info: host={0} port={1} code={2}",
                    host, port, r));

            foreach (KeyValuePair<string, string> entry in info)
            {
                if (entry.Key.Equals("statistics"))
                {
                    LogNameValueTokens(entry.Value);
                }
                else
                {
                    console.Log("{0}={1}", entry.Key, entry.Value);
                }
            }
        }

        /// <summary>
        /// Query namespace configuration.
        /// </summary>
        private void GetNamespaceConfig(string host, int port, string ns, ClLog log)
        {
            console.Log("Namespace Configuration");
            string filter = "namespace/" + ns;
            string val;
            CitrusleafResult r = CitrusleafClient.Info(host, port, filter, out val, log, null);

            if (r != CitrusleafResult.OK)
                throw new Exception(string.Format(
                    "Failed to get namespace info: host={0} port={1} namespace={2} code={3}",
                    host, port, ns, r));

            LogNameValueTokens(val);
        }

        private void LogNameValueTokens(string tokens)
        {
            string[] values = tokens.Split(';');

            foreach (string value in values)
            {
                console.Log(value);
            }
        }
    }
}
