﻿using System;
using System.Collections.Generic;
using net.citrusleaf;

namespace Citrusleaf.Demo
{
    class SetGet : Example
    {
        public SetGet(Console console)
            : base(console)
        {
        }

        /// <summary>
        /// Write and read a bin value.
        /// </summary>
        public override void RunExample(CitrusleafClient cs, string host, int port, string ns, string set, bool singleBin)
        {
            string key = "setgetkey";
            string bin = singleBin ? "" : "setgetbin";  // Single bin servers don't need a bin name.
            string value = "setgetvalue";

            CitrusleafResult r = cs.Set(ns, set, key, bin, value, null);

            if (r != CitrusleafResult.OK)
                throw new Exception(string.Format(
                    "Failed to set: ns={0} set={1} key={2} bin={3} value={4} code={5}", 
                    ns, set, key, bin, value, r));

            console.Info("Set successful: ns={0} set={1} key={2} bin={3} value={4}",
                ns, set, key, bin, value);

            object receivedValue;
            uint generation;

            r = cs.Get(ns, set, key, bin, out receivedValue, out generation, DEFAULT_TIMEOUT_MS);

            if (r != CitrusleafResult.OK)
                throw new Exception(string.Format(
                    "Failed to get: ns={0} set={1} key={2} bin={3} code={4}", 
                    ns, set, key, bin, r));

            if (value.Equals(receivedValue))
            {
                console.Info("Get successful: ns={0} set={1} key={2} bin={3} value={4}", 
                    ns, set, key, bin, receivedValue);
            }
            else
            {
                console.Error("Set/Get mismatch: Expected {0}. Received {1}.", 
                    value, receivedValue);
            }
        }
    }
}
