﻿using System;
using System.Collections.Generic;
using net.citrusleaf;
using System.Threading;

namespace Citrusleaf.Demo
{
    class Touch : Example
    {
        public Touch(Console console)
            : base(console)
        {
        }

        /// <summary>
        /// Touch a record. Touch resets the seconds interval to expiration.
        /// </summary>
        public override void RunExample(CitrusleafClient cs, string host, int port, string ns, string set, bool singleBin)
        {
            string key = "touchkey";
            string bin = singleBin ? "" : "touchbin";  // Single bin servers don't need a bin name.
            string value = "touchvalue";

            console.Info("Create record with 2 second expiration.");
            CitrusleafWriteInfo writeInfo = new CitrusleafWriteInfo(DEFAULT_TIMEOUT_MS);
            writeInfo.ExpirationSet(2);
            CitrusleafResult r = cs.Set(ns, set, key, bin, value, writeInfo);

            if (r != CitrusleafResult.OK)
                throw new Exception(string.Format(
                    "Failed to set: ns={0} set={1} key={2} bin={3} value={4} code={5}",
                    ns, set, key, bin, value, r));

            console.Info("Touch same record with 5 second expiration.");
            r = cs.Touch(ns, set, key, 5, DEFAULT_TIMEOUT_MS);

            if (r != CitrusleafResult.OK)
                throw new Exception(string.Format("Failed to touch: ns={0} set={1} key={2} code={3}", 
                    ns, set, key, r));

            console.Info("Sleep 3 seconds.");
            Thread.Sleep(3000);

            object received;
            uint generation;
            r = cs.Get(ns, set, key, bin, out received, out generation, DEFAULT_TIMEOUT_MS);

            if (r != CitrusleafResult.OK)
                throw new Exception(string.Format("Failed to get: ns={0} set={1} key={2} bin={3} code={4}",
                    ns, set, key, bin, r));

            console.Info("Success. Record still exists.");
            console.Info("Sleep 4 seconds.");
            Thread.Sleep(4000);

            r = cs.Get(ns, set, key, bin, out received, out generation, DEFAULT_TIMEOUT_MS);

            switch (r)
            {
                case CitrusleafResult.OK:
                    console.Error("Found record when it should have expired.");
                    break;

                case CitrusleafResult.KEY_NOT_FOUND_ERROR:
                    console.Info("Success. Record expired as expected.");
                    break;

                default:
                    throw new Exception(string.Format(
                        "Failed to get: ns={0} set={1} key={2} bin={3} code={4}", 
                        ns, set, key, bin, r));
            }
        }
    }
}
