using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace net.citrusleaf
{
    class Info
    {
        public static byte[] Worker(IPEndPoint ipe, byte[] request, ClLog log) 
        {
            byte[] h_buf = new byte[8];
            ulong h_long = (2L << 56) + (1L << 48);
            int offset;
            byte[] r_buf = null;

            Socket sock = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            CitrusleafSocket.Connect(sock, ipe, 2000);

            try
            {
                sock.NoDelay = true;
        
                if (request != null)
                {
                    h_long += (ulong)request.Length;
                    Buffer.set_htonll(h_long, h_buf, 0);
                    ArraySegment<byte>[] send_list = new ArraySegment<byte>[] {
                        new ArraySegment<byte>(h_buf), new ArraySegment<byte>(request) };

                    if (request.Length + 8 != sock.Send(send_list))
                    {
                        log.Log(CitrusleafClient.LogLevel.WARNING, " infom: warning could not write request: looks like "+ipe.ToString()+" has gone bad");
                        return (null);
                    }
                }
                else
                {
                    Buffer.set_htonll(h_long, h_buf, 0);
                    if (8 != sock.Send(h_buf))
                    {
                        log.Log(CitrusleafClient.LogLevel.WARNING, " info: warning could not write request: looks like " + ipe.ToString() + "  has gone bad");
                        return (null);
                    }
                }

                // read in header
                offset = 0;
                while (offset < 8)
                {
                    SocketError se;
                    int rlen_now = 0;
                    rlen_now = sock.Receive(h_buf, offset, 8 - offset, SocketFlags.None, out se);
                    if (se != SocketError.Success)
                    {
                        log.Log(CitrusleafClient.LogLevel.WARNING, " info: received socket error {0}. Returning SERVER_ERROR {1}", se.ToString(), ipe.ToString());
                        return (null);
                    }
                    if (rlen_now <= 0)
                    {
                        log.Log(CitrusleafClient.LogLevel.WARNING, " info: received 0 bytes. Returning SERVER_ERROR {0}", ipe.ToString());
                        return (null);
                    }
                    offset += rlen_now;
                }

                // check version and msg type
                if (h_buf[0] != 2 || h_buf[1] != 1)
                {
                    log.Log(CitrusleafClient.LogLevel.WARNING,"warning: received bad version in Info - unlikely");
                    return (null);
                }

                h_buf[0] = 0; h_buf[1] = 0;
                h_long = Buffer.get_ntohll(h_buf, 0);


                if (h_long != 0)
                {
                    offset = 0;
                    r_buf = new byte[h_long];
                    while (offset < (int)h_long)
                    {
                        int rlen_now = 0;
                        SocketError se;
                        rlen_now = sock.Receive(r_buf, offset, (int)h_long - offset, SocketFlags.None, out se);
                        if (se != SocketError.Success) return (null);
                        if (rlen_now <= 0)
                        {
                            log.Log(CitrusleafClient.LogLevel.WARNING, " info2 received 0 bytes. Returning SERVER_ERROR " + ipe.ToString());
                            return (null);
                        }
                        offset += rlen_now;
                    }
                }
            }
            finally
            {
                sock.Close();
            }

            return(r_buf);
        }

        // Multiple value request
        public static CitrusleafResult Get(IPEndPoint ipe, IList<string> request_sl, out IDictionary<string,string> response, ClLog log) 
        {
            response = null;
            byte[] request_buf = null;

            if (request_sl != null)
            {
                StringBuilder sb = new StringBuilder(200);
                foreach (string s in request_sl)
                {
                    sb.Append(s);
                    sb.Append("\n");
                }
                request_buf = Encoding.ASCII.GetBytes(sb.ToString());
            }
            byte[] response_buf = null;
            try
            {
                response_buf = Worker(ipe, request_buf, log);
            }
            catch (SocketException e)
            {
                if (e.ErrorCode == 0)
                {
                    log.Log(CitrusleafClient.LogLevel.DEBUG, "Failed to connect: " + ipe.ToString());
                }
                else
                {
                    log.Log(CitrusleafClient.LogLevel.WARNING, "Info " + ipe.ToString() + " socket exception {0}: {1}", e.ErrorCode, e.Message);
                }
                return (CitrusleafResult.SERVER_ERROR);
            }
            if (response_buf == null)
            {
                log.Log(CitrusleafClient.LogLevel.WARNING, "Info " + ipe.ToString() + ": no response back");
                return (CitrusleafResult.SERVER_ERROR);
            }
            response = new Dictionary<string, string>();
            int offset = 0;
            int start_tok = 0;
            string name = null;
            string response_str = Encoding.ASCII.GetString(response_buf);
            // log.DebugFormat("starting to parse: length {0}",response_buf.Length);
            while (offset < response_buf.Length)
            {
                if (response_buf[offset] == 9) {
                    name = Encoding.ASCII.GetString(response_buf, start_tok, offset - start_tok);
                    // log.DebugFormat("found tab offset {0} name {1}", offset, name);
                    start_tok = offset + 1;
                }
                else if ((response_buf[offset] == 10) ||
                         (response_buf[offset] == 13))
                {
                    if (name != null)
                    {
                        string value = Encoding.ASCII.GetString(response_buf, start_tok, offset - start_tok);
                        // log.DebugFormat("found crlf: offset {0} name {1} value {2}", offset, name, value);
                        response[name] = value;
                        name = null;
                    }
                    else
                    { 
                        // found a name without a value
                        name = Encoding.ASCII.GetString(response_buf, start_tok, offset - start_tok);
                        // log.DebugFormat("found crlf: name wo value: offset {0} name {1}", offset, name);
                        response.Add(name, null);
                    }

                    start_tok = offset + 1;
                }

                offset++;
            }
            return (CitrusleafResult.OK);
        }

        // Single value request
        public static CitrusleafResult Get(IPEndPoint ipe, string request, out string response, ClLog log)
        {
            response = null;
            if (request[request.Length-1] != '\n') request += "\n";
            byte[] request_buf = Encoding.ASCII.GetBytes(request);

            byte[] response_buf = null;
            try
            {
                response_buf = Worker(ipe, request_buf, log);
            }
            catch (SocketException e)
            {
                if (e.ErrorCode == 0)
                {
                    log.Log(CitrusleafClient.LogLevel.DEBUG, "Failed to connect: " + ipe.ToString());
                }
                else
                {
                    log.Log(CitrusleafClient.LogLevel.WARNING, "Info " + ipe.ToString() + " socket exception {0}: {1}", e.ErrorCode, e.Message);
                }
                return (CitrusleafResult.SERVER_ERROR);
            }
            if (response_buf == null)
            {
                log.Log(CitrusleafClient.LogLevel.WARNING, "Info " + ipe.ToString() + ": no response back");
                return (CitrusleafResult.SERVER_ERROR);
            }

            // format: name\tvalue\n - so find the stuff between the \t and \n

            // way1:
            response = Encoding.ASCII.GetString(response_buf);
            int tab_pos = response.IndexOf('\t');
            response = response.Substring(tab_pos + 1, response.Length - (tab_pos + 2));

            // way2:
            //for (int i = 0; i < response_buf.Length; i++)
            //{
            //    if (response_buf[i] == '\t')
            //    {
            //        response = Encoding.ASCII.GetString(response_buf, i + 1, response_buf.Length - (i + 2));
            //        break;
            //    }
            //}

            return (CitrusleafResult.OK);
        }
    }
}
