﻿// RTA (Real-time attribution) product specific classes.
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace net.citrusleaf
{
    /// <summary>
    /// This class is used to specify how append transactions are performed in the Citrusleaf RTA product.
    /// </summary>
    public class ClAppendWriteOptions
    {
        /// <summary>
        /// Grouping attribute that will be appended to a segment.  This tag can be referenced in queries.
        /// </summary>
        public string tag;
        /// <summary>
        /// Seconds before object expiration.
        /// </summary>
        public uint expiration;
        /// <summary>
        /// Object creation time.
        /// </summary>
        public DateTime creation_time;

        /// <summary>
        /// Construct append transaction parameters.
        /// </summary>
        /// <param name="tag">Grouping attribute that will be appended to a value.</param>
        /// <param name="expiration">Seconds before object expiration.</param>
        /// <param name="creation_time">Object creation time.</param>
        public ClAppendWriteOptions(string tag, uint expiration, DateTime creation_time)
        {
            this.tag = tag;
            this.expiration = expiration;
            this.creation_time = creation_time;
        }
    }

    /// <summary>
    /// This class handles query filters for the Citrusleaf RTA product.
    /// </summary>
    public class ClAppendQueryFilter
    {
        /// <summary>
        /// Return segments created at or after the specified time.
        /// </summary>
        public DateTime start_time;
        /// <summary>
        /// Return segments created before the specified time.
        /// </summary>
        public DateTime end_time;
        /// <summary>
        /// Return segments matching this segment tag.
        /// </summary>
        public string tag;

        /// <summary>
        /// Construct query filters.
        /// </summary>
        public ClAppendQueryFilter(DateTime start_time, DateTime end_time, string tag)
        {
            this.start_time = start_time;
            this.end_time = end_time;
            this.tag = tag;
        }
    }
}
