﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace net.citrusleaf
{
    /// <summary>
    /// Result code indicating status of requested action.
    /// </summary>
    public enum CitrusleafResult
    {
        /// <summary>
        /// Request has succeeded.
        /// </summary>
        OK = 0,
        /// <summary>
        /// Server has not yet returned a result.
        /// </summary>
        NOT_SET,
        /// <summary>
        /// Unknown server failure.
        /// </summary>
        SERVER_ERROR,
        /// <summary>
        /// Memory or other fault in the client.
        /// </summary>
        CLIENT_ERROR,
        /// <summary>
        /// Requested key does not exist.
        /// </summary>
        KEY_NOT_FOUND_ERROR,
        /// <summary>
        /// Write failed because unique key already exists.
        /// </summary>
        KEY_FOUND_ERROR,
        /// <summary>
        /// Requested bin does not exist.  Only returned by a bin-specific get.
        /// </summary>
        BIN_NOT_FOUND_ERROR,
        /// <summary>
        /// Write failed because unique bin already exists.
        /// </summary>
        BIN_FOUND_ERROR,
        /// <summary>
        /// The generation specified in the write is not the same generation on the server.  
        /// Bin value(s) have been modified by another process.
        /// </summary>
        GENERATION_ERROR,
        /// <summary>
        /// The server detected a problem in your input.
        /// </summary>
        PARAMETER_ERROR,
        /// <summary>
        /// Request has timed out waiting for a response.
        /// </summary>
        TIMEOUT,
        /// <summary>
        /// No servers were found.
        /// </summary>
        NO_HOSTS,
        /// <summary>
        /// Server is not accepting requests.
        /// </summary>
        SERVER_NOT_AVAILABLE,
        /// <summary>
        /// Error communicating with server.
        /// </summary>
        UNKNOWN_ERROR,
        /// <summary>
        /// Server has run out of memory.
        /// </summary>
        SERVER_MEM_ERROR,
        /// <summary>
        /// Scan has been terminated before completion.
        /// </summary>
        SCAN_TERMINATED,
        /// <summary>
        /// Expected Cluster ID was not received.
        /// </summary>
        CLUSTER_KEY_MISMATCH,
        /// <summary>
        /// XDS product is not available.
        /// </summary>
        NO_XDS,
        /// <summary>
        /// Operation is not supported with configured bin type (single-bin or multi-bin).
        /// </summary>
        BIN_TYPE_ERROR,
        /// <summary>
        /// Record size exceeds maximum limit.
        /// </summary>
        RECORD_TOO_BIG,
        /// <summary>
        /// Too many concurrent operations on the same record.
        /// </summary>
        KEY_BUSY
    };
}
