﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace net.citrusleaf
{
    /// <summary>
    /// Citrusleaf socket wrapper.
    /// </summary>
    public class CitrusleafSocket
    {
        public DateTime timestamp;
        public Socket s;

        public CitrusleafSocket(Socket socket)
        {
            this.s = socket;
            this.timestamp = DateTime.Now;
        }

        public void Dispose()
        {
            s.Dispose();
        }

        public void Connect(IPEndPoint ipe, int timeoutMillis)
        {
            Connect(s, ipe, timeoutMillis);
        }

        /// <summary>
        /// Socket connect with timeout.
        /// </summary>
        /// <param name="socket">Socket.</param>
        /// <param name="ipe">Socket endpoint</param>
        /// <param name="timeoutMillis">Timeout in milliseconds</param>
        public static void Connect(Socket socket, IPEndPoint ipe, int timeoutMillis)
        {
            IAsyncResult result = socket.BeginConnect(ipe, null, null);
            WaitHandle wait = result.AsyncWaitHandle;

            try
            {
                if (wait.WaitOne(timeoutMillis))
                {
                    socket.EndConnect(result);
                }
                else
                {
                    socket.Close();
                    throw new SocketException();
                }
            }
            finally
            {
                wait.Close();
            }
        }

        /// <summary>
        /// GetHostAddresses with timeout.
        /// </summary>
        /// <param name="host">Host name.</param>
        /// <param name="timeoutMillis">Timeout in milliseconds</param>
        public static IPAddress[] GetHostAddresses(string host, int timeoutMillis)
        {
            IAsyncResult result = Dns.BeginGetHostAddresses(host, null, null);
            WaitHandle wait = result.AsyncWaitHandle;

            try
            {
                if (wait.WaitOne(timeoutMillis))
                {
                    IPAddress[] addresses = Dns.EndGetHostAddresses(result);
                    return addresses;
                }
                else
                {
                    throw new Exception("Failed to resolve " + host);
                }
            }
            finally
            {
                wait.Close();
            }
        }
    }
}
